#!/usr/bin/env python3
# coding: utf8


from functools import partial
import requests


class TeleBot(object):
    def __init__(self, access_token):
        self._access_token = access_token

    def __getattr__(self, method):
        return partial(self._call, method)

    def __call__(self, method, **params):
        return getattr(self, method)(**params)

    def _call(self, method, **params):
        url = f'https://api.telegram.org/bot{self._access_token}/{method}'
        return requests.post(url, params).json()

    def get_updates(self, **params):
        url = f'https://api.telegram.org/bot{self._access_token}/getUpdates'
        if 'timeout' in params:
            return requests.post(url, params, timeout=params['timeout']+5).json()
        return requests.post(url, params).json()

    def send_message(self, chat_id, message):
        url = f'https://api.telegram.org/bot{self._access_token}/sendMessage'
        params = {'chat_id': chat_id, 'text': message}
        return requests.post(url, params).json()

    def reply_message(self, chat_id, message, reply_to_message_id):
        url = f'https://api.telegram.org/bot{self._access_token}/sendMessage'
        params = {'chat_id': chat_id, 'text': message, 'reply_to_message_id': reply_to_message_id}
        return requests.post(url, params).json()

    def delete_message(self, chat_id, message_id):
        url = f'https://api.telegram.org/bot{self._access_token}/deleteMessage'
        params = {'chat_id': chat_id, 'message_id': message_id}
        return requests.post(url, params).json()


if __name__ == '__main__':
    bot = TeleBot('456941934:AAGZMmXJE4VyLagIkVY7qMG0doASxU7f8ac')
    #bot.send_message(161680036, 'Hello')
    bot.reply_message(161680036, 'reply for your message', 16394)


from string import ascii_letters, digits
from typing import Union

from cryptography.fernet import Fernet

alphabet_list = ascii_letters + digits


def caesar_code(text: str, shift: int):
    shift_text = ''

    for c in text:
        if c not in alphabet_list:
            shift_text += c
            continue

        i = (alphabet_list.index(c) + shift) % len(alphabet_list)
        shift_text += alphabet_list[i]

    return shift_text


def encode_password(text: str):
    for offcet in range(1, 10):
        text = caesar_code(text, shift=offcet)
    return text


def decode_password(text: str):
    for offcet in range(9, 0, -1):
        text = caesar_code(text, shift=-offcet)
    return text
    

def get_key():
    return Fernet.generate_key()


def get_cipher(cipher_key: Union[bytes, str]):
    return Fernet(cipher_key)


def encrypt_str(text: Union[bytes, str], cipher_: Fernet, get_str: bool = True):
    if isinstance(text, str):
        text = text.encode('utf-8')
    text_enc = cipher_.encrypt(text)
    if get_str:
        text_enc = text_enc.decode('utf-8')
    return text_enc


def decrypt_str(text: Union[bytes, str], cipher_: Fernet, get_str: bool = True):
    if isinstance(text, str):
        text = text.encode('utf-8')
    text_dec = cipher_.decrypt(text)
    if get_str:
        text_dec = text_dec.decode('utf-8')
    return text_dec


def write_key(file_name: str, cipher_key: bytes):
    with open(file_name, 'wb') as f:
        f.write(cipher_key)


def read_key(file_name: str):
    try:
        with open(file_name, 'rb') as f:
            cipher_key = f.read()
    except:
        cipher_key = None
    return cipher_key


if __name__ == '__main__':
    # TEST
    cipher_key = get_key()
    cipher = get_cipher(cipher_key)
    text = 'qwerty1234!!'
    text_enc = encrypt_str(text, cipher)
    write_key('cipher_key', cipher_key)
    new_cipher_key = read_key('cipher_key')
    new_cipher = get_cipher(new_cipher_key)
    text_dec = decrypt_str(text_enc, new_cipher)
    print(text)
    print(text_dec)



try:
    from transliterate import translit, get_available_language_codes
    from transliterate.discover import autodiscover
    from transliterate.base import TranslitLanguagePack, registry
except ImportError:
    raise ImportError("transliterate isn't installed. Run: pip install -U transliterate")

#autodiscover()


class ExtendedRussianLangPack(TranslitLanguagePack):
    language_code = 'ru_ext'
    language_name = 'Russian extended'
    mapping = (
        'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',
        'абцдефгхийклмнопкрстуввкызАБЦДЕФГХИЙКЛМНОПКРСТУВВКЫЗ',
    )
    reversed_specific_mapping = (
        'вВёЁйЙкКэЭыЫьЬъЪ',
        "vVeEiIkKeEyY''''"
    )
    pre_processor_mapping = {
        'shch': 'щ',
        'ya': 'я',
        'ia': 'я',
        'yu': 'ю',
        'iu': 'ю',
        'ch': 'ч',
        'sh': 'ш',
        'x': 'кс',
        'zh': 'ж',
        'ts': 'ц',
        'kh': 'х',
        'SHCH': 'Щ',
        'YA': 'Я',
        'IA': 'Я',
        'YU': 'Ю',
        'IU': 'Ю',
        'CH': 'Ч',
        'SH': 'Ш',
        'X': 'КС',
        'ZH': 'Ж',
        'TS': 'Ц',
        'KH': 'Х',
    }
    reversed_specific_pre_processor_mapping = {
        'ж': 'zh',
        'х': 'kh',
        'ц': 'ts',
        'ч': 'ch',
        'ш': 'sh',
        'щ': 'shch',
        'ю': 'iu',
        'я': 'ia',
        'Ж': 'ZH',
        'Х': 'KH',
        'Ц': 'TS',
        'Ч': 'CH',
        'Ш': 'SH',
        'Щ': 'SHCH',
        'Ю': 'IU',
        'Я': 'IA',
    }


class RussianAftnLangPack(TranslitLanguagePack):
    language_code = 'ru_aftn'
    language_name = 'Russian aftn'
    mapping = (
        'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',
        'абцдефгхийклмнопщрстужвкызАБЦДЕФГХИЙКЛМНОПЩРСТУЖВКЫЗ',
    )
    reversed_specific_mapping = (
        'вВёЁжЖйЙхХщЩъЪыЫьЬэЭЧкК',
        'wWeEvViIhHqQxXyYxXeE4kK'
    )
    pre_processor_mapping = {
        'ya': 'я',
        'ia': 'я',
        'yu': 'ю',
        'iu': 'ю',
        'ch': 'ч',
        'sh': 'ш',
        'x': 'кс',
        'YA': 'Я',
        'IA': 'Я',
        'YU': 'Ю',
        'IU': 'Ю',
        'CH': 'Ч',
        'SH': 'Ш',
        'X': 'КС',
    }
    reversed_specific_pre_processor_mapping = {
        'ч': 'ch',
        'ш': 'sh',
        'ю': 'iu',
        'я': 'ia',
        'Ш': 'SH',
        'Ю': 'IU',
        'Я': 'IA',
    }


registry.register(ExtendedRussianLangPack, force=True)
registry.register(RussianAftnLangPack, force=True)


from divinegift import logger
from divinegift.errors import SenderNotInitializedError

try:
    from pyzabbix import ZabbixMetric, ZabbixSender
except ImportError:
    raise ImportError("py-zabbix isn't installed. Run: pip install -U py-zabbix")


class ZabbixAgent:
    def __init__(self, zabbix_server=None, zabbix_port=10051):
        self.sender = None
        self.sender = self.init_sender(zabbix_server, zabbix_port) if zabbix_server is not None else None

    def init_sender(self, zabbix_server, zabbix_port=10051):
        if self.sender is None:
            return ZabbixSender(zabbix_server, zabbix_port)
        else:
            return self.sender

    def send(self, host, key, value):
        if self.sender is None:
            raise SenderNotInitializedError('Initialize sender first!')

        try:
            result = self.sender.send([ZabbixMetric(host=host, key=key, value=value)])
        except Exception as ex:
            result = {}
        logger.log_debug(result)
        return result
